@@ -3,11 +3,11 @@
 ;; Copyright (C) 2006-2009, 2011-2012, 2015-2018
 ;;   Phil Hagelberg, Doug Alcorn, Will Farrington, Chen Bin
 ;;
-;; Version: 5.7.3
+;; Version: 5.7.7
 ;; Author: Phil Hagelberg, Doug Alcorn, and Will Farrington
 ;; Maintainer: Chen Bin <chenbin.sh@gmail.com>
 ;; URL: https://github.com/technomancy/find-file-in-project
-;; Package-Requires: ((ivy "0.10.0") (emacs "24.3"))
+;; Package-Requires: ((ivy "0.10.0") (emacs "24.4"))
 ;; Created: 2008-03-18
 ;; Keywords: project, convenience
 ;; EmacsWiki: FindFileInProject
@@ -152,6 +152,7 @@
 
 (require 'diff-mode)
 (require 'windmove)
+(require 'subr-x)
 
 (defvar ffip-use-rust-fd nil "Use use fd instead of find.")
 
@@ -170,11 +171,16 @@
   "Dictionary to look up windows split ratio.
 Used by `ffip-split-window-horizontally' and `ffip-split-window-vertically'.")
 
+(defvar ffip-filename-history nil)
+
 (defvar ffip-strip-file-name-regex
   "\\(\\.mock\\|\\.test\\|\\.mockup\\)"
   "Strip file name to get minimum keyword with this regex.
 It's used by `find-file-with-similar-name'.")
 
+(defvar ffip-split-window-without-asking-for-keyword nil
+  "`ffip-split-window-horizontally' or `ffip-split-window-vertically' don't ask keyword.")
+
 (defvar ffip-diff-find-file-before-hook nil
   "Hook before `ffip-diff-find-file' move focus out of *ffip-diff* buffer.")
 
@@ -195,11 +201,22 @@ The file path is passed to the hook as the first argument.")
         standard-output
       (shell-command command t))))
 
+(defun ffip-nonempty-lines (s)
+  "Return non empty lines."
+  (split-string s "[\r\n]+" t))
+
+(defun ffip-diff-git-versions ()
+  "List all versions of code under Git."
+  (let* ((git-cmd (concat "git --no-pager log --date=short --pretty=format:'%h|%ad|%s|%an' "
+                          buffer-file-name)))
+    (nconc (ffip-nonempty-lines (shell-command-to-string "git branch --no-color --all"))
+           (ffip-nonempty-lines (shell-command-to-string git-cmd)))))
+
 ;;;###autoload
 (defun ffip-git-diff-current-file ()
   "Run 'git diff version:current-file current-file'."
   (let* ((default-directory (locate-dominating-file default-directory ".git"))
-         (line (ivy-read "diff current file:" (my-git-versions))))
+         (line (ivy-read "diff current file:" (ffip-diff-git-versions))))
     (ffip-shell-command-to-string (format "git --no-pager diff %s:%s %s"
                                      (replace-regexp-in-string "^ *\\*? *" "" (car (split-string line "|" t)))
                                      (file-relative-name buffer-file-name default-directory)
@@ -208,7 +225,7 @@ The file path is passed to the hook as the first argument.")
 (defun ffip-git-diff-project()
   "Run 'git diff version' in project."
   (let* ((default-directory (locate-dominating-file default-directory ".git"))
-         (line (ivy-read "diff current file:" (my-git-versions)))
+         (line (ivy-read "diff current file:" (ffip-diff-git-versions)))
          (version (replace-regexp-in-string "^ *\\*? *" "" (car (split-string line "|" t)))))
     (ffip-shell-command-to-string (format "git --no-pager diff %s" version))))
 
@@ -385,18 +402,29 @@ This overrides variable `ffip-project-root' when set.")
 (defvar ffip-find-relative-path-callback 'ffip-copy-without-change
   "The callback after calling `find-relative-path'.")
 
+(defun ffip--some (predicate seq)
+  "Return if PREDICATE is of true of any element of SEQ"
+  (let* (elem rlt)
+    (while (and (setq elem (car seq))
+                (not rlt))
+      (setq seq (cdr seq))
+      (setq rlt (funcall predicate elem)))
+    rlt))
+
 ;;;###autoload
 (defun ffip-project-root ()
   "Return the root of the project."
-  (let ((project-root (or ffip-project-root
-                          (if (functionp ffip-project-root-function)
-                              (funcall ffip-project-root-function)
-                            (if (listp ffip-project-file)
-                                (cl-some (apply-partially 'locate-dominating-file
-                                                       default-directory)
-                                      ffip-project-file)
-                              (locate-dominating-file default-directory
-                                                      ffip-project-file))))))
+  (let* ((project-root (or ffip-project-root
+                           (cond
+                            ((functionp ffip-project-root-function)
+                             (funcall ffip-project-root-function))
+                            ((listp ffip-project-file)
+                             (ffip--some (apply-partially 'locate-dominating-file
+                                                          default-directory)
+                                         ffip-project-file))
+                            (t
+                             (locate-dominating-file default-directory
+                                                     ffip-project-file))))))
     (or (and project-root (file-name-as-directory project-root))
         (progn
           (message "Since NO project was found, use `default-directory' instead.")
@@ -707,11 +735,7 @@ DIRECTORY-TO-SEARCH specify the root directory to search."
 If OPEN-ANOTHER-WINDOW is t, the results are displayed in a new window.
 If FIND-DIRECTORY is t, only search directories.  FN is callback.
 This function is the API to find files."
-  (let* (cands
-         lnum
-         file
-         root)
-
+  (let* (cands lnum file root)
     ;; extract line num if exists
     (when (and keyword (stringp keyword)
                (string-match "^\\(.*\\):\\([0-9]+\\):?$" keyword))
@@ -752,9 +776,16 @@ This function is the API to find files."
 (defun ffip-read-keyword ()
   "Read keyword from selected text or user input."
   (let* ((hint (if ffip-use-rust-fd "Enter regex (or press ENTER):"
-                 "Enter keyword (or press ENTER):")))
-    (if (region-active-p) (ffip--read-selected)
-      (read-string hint))))
+                 "Enter keyword (or press ENTER):"))
+         rlt)
+    (cond
+     ((region-active-p)
+      (setq ffip-filename-history (add-to-list 'ffip-filename-history
+                                               (ffip--read-selected)))
+      (setq rlt (ffip--read-selected)))
+     (t
+      (setq rlt (read-from-minibuffer hint nil nil nil 'ffip-filename-history))))
+    (if rlt (string-trim rlt) rlt)))
 
 ;;;###autoload
 (defun ffip-create-project-file ()
@@ -860,13 +891,22 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
      (t
       (message "No file name is provided.")))))
 
+(defun ffip-parent-directory (level directory)
+  "Return LEVEL up parent directory of DIRECTORY."
+  (let* ((rlt directory))
+    (while (and (> level 0) (not (string= "" rlt)))
+      (setq rlt (file-name-directory (directory-file-name rlt)))
+      (setq level (1- level)))
+    (if (string= "" rlt) (setq rlt nil))
+    rlt))
+
 ;;;###autoload
-(defun find-file-in-current-directory (&optional open-another-window)
-  "Like `find-file-in-project'.  But search only in current directory.
-IF OPEN-ANOTHER-WINDOW is t, results are displayed in new window."
+(defun find-file-in-current-directory (&optional level)
+  "Search fil in current directory or LEVEL up parent directory."
   (interactive "P")
-  (let* ((ffip-project-root default-directory))
-    (find-file-in-project open-another-window)))
+  (unless level (setq level 0))
+  (let* ((ffip-project-root (ffip-parent-directory level default-directory)))
+    (find-file-in-project nil)))
 
 ;;;###autoload
 (defun find-file-in-project-by-selected (&optional open-another-window)
@@ -985,7 +1025,9 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
   "Use SPLIT-FN to split window and focus on new window by MV-FN.
 Window split in RATIO."
   (let* (ratio-val
-         (cands (ffip-project-search (ffip-read-keyword) nil))
+         (keyword (if ffip-split-window-without-asking-for-keyword ""
+                    (ffip-read-keyword)))
+         (cands (ffip-project-search keyword nil))
          (file (if (= 1 (length cands)) (ffip-path (car cands))
                  (ffip-path (ffip-completing-read "Find file: " cands))))
          (buf (if (and file (file-exists-p file)) (find-file-noselect file)
@@ -1041,14 +1083,17 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
         (setq alnum (string-to-number (match-string 1)))
         (setq blnum (string-to-number (match-string 3)))))
 
-    (if (and (> (length files) 1)
-             (string= (nth 0 files) (nth 1 files)))
-        (ffip-find-files (nth 0 files)
-                         open-another-window
-                         nil
-                         (lambda (opened-file)
-                           ;; use line number in new file since there is only one file name candidate
-                           (ffip--forward-line blnum)))
+    (cond
+     ((and (> (length files) 1)
+           (string= (nth 0 files) (nth 1 files)))
+      (ffip-find-files (nth 0 files)
+                       open-another-window
+                       nil
+                       `(lambda (opened-file)
+                          ;; use line number in new file since there
+                          ;; is only one file name candidate
+                          (ffip--forward-line ,blnum))))
+     (t
       (run-hook-with-args 'ffip-diff-find-file-before-hook)
       (ffip-find-files files
                        open-another-window
@@ -1058,7 +1103,7 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
                           ((string= (file-name-nondirectory opened-file) (nth 0 files))
                            (ffip--forward-line alnum))
                           (t
-                           (ffip--forward-line blnum))))))))
+                           (ffip--forward-line blnum)))))))))
 
 (defvar ffip-diff-mode-map
   (let ((map (make-sparse-keymap)))
